@@ -4,7 +4,7 @@
  * GChemPaint library
  * bond.cc 
  *
- * Copyright (C) 2001-2004
+ * Copyright (C) 2001-2005
  *
  * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -225,13 +225,20 @@ Object* gcpBond::GetAtomAt(double x, double y, double z)
 	return NULL;
 }
 
-bool gcpBond::SaveNode(xmlDocPtr xml, xmlNodePtr node)
+bool gcpBond::SaveNode (xmlDocPtr xml, xmlNodePtr node)
 {
-	switch(m_type)
-	{
-		case UpBondType: xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"up"); break;
-		case DownBondType: xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"down"); break;
-		case UndeterminedBondType: xmlNewProp(node, (xmlChar*)"type", (xmlChar*)"undetermined"); break;
+	switch (m_type) {
+	case UpBondType:
+		xmlNewProp (node, (xmlChar*) "type", (xmlChar*) "up");
+		break;
+	case DownBondType:
+		xmlNewProp (node, (xmlChar*) "type", (xmlChar*) "down");
+		break;
+	case UndeterminedBondType:
+		xmlNewProp (node, (xmlChar*) "type", (xmlChar*) "undetermined");
+		break;
+	default:
+		break;
 	}
 	return true;
 }
@@ -352,6 +359,7 @@ void gcpBond::SetSelected(GtkWidget* w, int state)
 		case SelStateSelected: color = SelectColor; break;
 		case SelStateUpdating: color = AddColor; break;
 		case SelStateErasing: color = DeleteColor; break;
+		default: color = Color; break;
 	}
 	void *obj = g_object_get_data(G_OBJECT(group), "path");
 	switch (GetType())
@@ -376,9 +384,8 @@ void gcpBond::Add(GtkWidget* w)
 	if (!(pAtom1 = GetAtom(1))) return;
 	unsigned char order = GetOrder();
 	if (order == 0) return;
-	double x1, y1, x2, y2;
 	GnomeCanvasGroup* group;
-	GnomeCanvasItem* item;
+	GnomeCanvasItem* item = NULL;
 	pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
 	group = GNOME_CANVAS_GROUP(gnome_canvas_item_new(pData->Group, gnome_canvas_group_ext_get_type(), NULL));
 	g_signal_connect(G_OBJECT(group), "event", G_CALLBACK(on_event), w);
@@ -459,6 +466,7 @@ bool gcpBond::ReplaceAtom(gcpAtom* oldAtom, gcpAtom* newAtom)
 		if (m_Begin && m_End)
 			m_Begin->AddBond(this);
 	}
+	return true;
 }
 
 GnomeCanvasPathDef* gcpBond::BuildPathDef (gcpWidgetData* pData)
