@@ -4,9 +4,9 @@
  * GChemPaint library
  * document.cc 
  *
- * Copyright (C) 2001-2004
+ * Copyright (C) 2001-2005
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -20,19 +20,20 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
-#include <mol.h>
-#undef PACKAGE
-#undef PACKAGE_BUGREPORT
-#undef PACKAGE_NAME
-#undef PACKAGE_STRING
-#undef PACKAGE_TARNAME
-#undef PACKAGE_VERSION
-#undef VERSION
-
+#ifndef HAS_OPENBABEL_2
+#	include <mol.h>
+#	undef PACKAGE
+#	undef PACKAGE_BUGREPORT
+#	undef PACKAGE_NAME
+#	undef PACKAGE_STRING
+#	undef PACKAGE_TARNAME
+#	undef PACKAGE_VERSION
+#	undef VERSION
+#endif
 #include "gchempaint-config.h"
 #include "application.h"
 #include "view.h"
@@ -44,12 +45,23 @@
 #include "text.h"
 #include <locale.h>
 #include <unistd.h>
+#include <libgen.h>
 #include <gtk/gtk.h>
-#include <iostream>
+#include <libgnomevfs/gnome-vfs-file-info.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
+#ifdef HAS_OPENBABEL_2
+#	include <openbabel/mol.h>
+#	include <openbabel/obconversion.h>
+#else
+#	include <sstream>
+#endif
 
 using namespace OpenBabel;
 using namespace std;
 
+SignalId OnChangedSignal;
+SignalId OnDeleteSignal;
+
 gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone): Document()
 {
 	m_pApp = App;
@@ -60,7 +72,11 @@ gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone): Document()
 	m_pView = new gcpView(this, !StandAlone);
 	m_bDirty = m_bIsLoading = m_bUndoRedo = false;
 	m_bWriteable = true;
+#ifdef HAS_OPENBABEL_2
+	m_FileType = gcp_ft_Normal;
+#else
 	m_FileType = UNDEFINED;
+#endif
 	m_DocPropDlg = NULL;
 	g_date_set_time(&CreationDate, time(NULL));
 	g_date_clear(&RevisionDate, 1);
@@ -72,23 +88,40 @@ gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone): Document()
 	else m_mail = NULL;
 	m_comment = NULL;
 	m_pCurOp = NULL;
+	m_bReadOnly = false;
 	SetActive();
 }
 
-gcpDocument::~gcpDocument()
+gcpDocument::~gcpDocument ()
 {
-	if (m_pCurOp) delete m_pCurOp;
-	if (m_DocPropDlg) m_DocPropDlg->Destroy();
-	if (m_pView) delete m_pView;
-	if (m_filename) g_free(m_filename);
-	if (m_title) g_free(m_title);
-	if (m_label) g_free(m_label);
-	if (m_author) g_free(m_author);
-	if (m_mail) g_free(m_mail);
-	if (m_comment) g_free(m_comment);
-	map<string, Object*>::iterator it;
-	while (HasChildren()) delete GetFirstChild(it);
-	m_pApp->RemoveDocument(this);
+	m_bIsLoading = true;
+	if (m_pCurOp)
+		delete m_pCurOp;
+	m_pCurOp = NULL;
+	if (m_DocPropDlg)
+		m_DocPropDlg->Destroy();
+	if (m_filename)
+		g_free (m_filename);
+	if (m_title)
+		g_free (m_title);
+	if (m_label)
+		g_free (m_label);
+	if (m_author)
+		g_free (m_author);
+	if (m_mail)
+		g_free (m_mail);
+	if (m_comment)
+		g_free (m_comment);
+	map<string, Object *>::iterator it;
+	Object *obj;
+	while (HasChildren ()) {
+		obj = GetFirstChild (it);
+		obj->Lock ();
+		Remove (obj);
+	}
+	m_pApp->RemoveDocument (this);
+	if (m_pView)
+		delete m_pView;
 }
 
 GtkWidget* gcpDocument::GetWidget()
@@ -108,43 +141,28 @@ const gchar* gcpDocument::GetLabel()
 	return m_label;
 }
 
-void gcpDocument::SetFileName(const gchar* Name, const gchar* ext)
+void gcpDocument::SetFileName (const gchar* Name, const gchar* ext)
 {
-	if (m_filename) g_free(m_filename);
-	if (*Name == '/') m_filename = g_strdup(Name);
-	else
-	{
-		gchar* curdir = g_get_current_dir();
-		m_filename = g_strconcat(curdir, "/", Name, NULL);
-		g_free(curdir);
-	}
-	int i = strlen(m_filename) - 1;
-	while ((m_filename[i] != '/') && (i >= 0)) i--;
-	if (i >=0) 
-	{
-		m_filename[i] = 0;
-		chdir(m_filename);
-		m_filename[i] = '/';
-	}
+	if (m_filename) g_free (m_filename);
+	m_filename = g_strdup (Name);
+	m_pApp->SetCurDir (dirname (m_filename));
+	g_free (m_filename);
+	m_filename = g_strdup (Name);
+	int i = strlen (m_filename) - 1;
+	while ((m_filename[i] != '/') && (i >= 0))
+		i--;
 	i++;
-	int j = strlen(m_filename) - 1;
-	while ((i < j) && (m_filename[j] != '.')) j--;
+	int j = strlen (m_filename) - 1;
+	while ((i < j) && (m_filename[j] != '.'))
+		j--;
 	if (m_label) g_free(m_label);
-	if (strcmp(m_filename + j, ext)) m_label = g_strdup(m_filename + i);
-	else m_label = g_strndup(m_filename + i, j - i);
-	if (m_pView) m_pView->UpdateLabel(m_label);
+	if (strcmp(m_filename + j, ext))
+		m_label = g_strdup (m_filename + i);
+	else
+		m_label = g_strndup (m_filename + i, j - i);
+	if (m_pView) m_pView->UpdateLabel (m_label);
 }
 
-/*void gcpDocument::FillCanvas()
-{
-	AnalTree();
-	std::map<std::string, gcpBond*>::iterator j;
-	for (j = m_Bonds.begin(); j != m_Bonds.end(); j++) m_pView->AddBond((*j).second);
-/*	std::map<std::string, gcpAtom*>::iterator i;
-	for (i = m_Atoms.begin(); i != m_Atoms.end(); i++) m_pView->AddAtom((*i).second);
-}*/
-
-
 void gcpDocument::BuildBondList(list<gcpBond*>& BondList, Object* obj)
 {
 	Object* pObject;
@@ -172,7 +190,7 @@ bool gcpDocument::ImportOB(OBMol& Mol)
 	m_title = g_strdup(Mol.GetTitle());
 	OBAtom *atom;
 	gcpAtom* pAtom;
-	double xmin, xmax, ymin, ymax, x, y;
+	double xmin = 0., xmax = 0., ymin = 0., ymax = 0., x, y;
 	bool valid = false;
 	vector<OBNodeBase*>::iterator i;
 	for (atom = Mol.BeginAtom(i); atom; atom = Mol.NextAtom(i))
@@ -233,6 +251,10 @@ bool gcpDocument::ImportOB(OBMol& Mol)
 			AddBond(pBond);
 		}
 	}
+	m_pApp->ActivateMenu ("Image", HasChildren ());
+	m_pView->Update (this);
+	Update ();
+	m_pView->EnsureSize ();
 	return true;
 }
 
@@ -247,7 +269,11 @@ void gcpDocument::BuildAtomTable(map<string, unsigned>& AtomTable, Object* obj,
 
 void gcpDocument::ExportOB()
 {
+#ifdef HAS_OPENBABEL_2
+	OBMol Mol;
+#else
 	OBMol Mol(UNDEFINED, (io_type) m_FileType);
+#endif
 	map<string, unsigned>::iterator i;
 	map<string, unsigned> AtomTable;
 	list<gcpBond*> BondList;
@@ -270,7 +296,6 @@ void gcpDocument::ExportOB()
 		Mol.AddAtom(Atom);
 		Atom.Clear();
 	}
-	OBAtom* pAtom;
 	BuildBondList(BondList, this);
 	list<gcpBond*>::iterator j;
 	int start, end, order, flag;
@@ -295,19 +320,33 @@ void gcpDocument::ExportOB()
 	Mol.EndModify();
 	try
 	{
-		ofstream ofs;
-		ofs.open(m_filename);
-		if (!ofs) throw (int) 1;
+		ostringstream ofs;
+		GnomeVFSHandle *handle = NULL;
+		GnomeVFSFileSize n;
+		GnomeVFSResult res;
+		if ((res = gnome_vfs_create(&handle, m_filename, GNOME_VFS_OPEN_WRITE, true, 0644)) != GNOME_VFS_OK)
+			throw (int) res;
 		old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
 		setlocale(LC_NUMERIC, "C");
+#ifdef HAS_OPENBABEL_2
+		OBConversion Conv;
+		OBFormat* pOutFormat = Conv.FormatFromExt (m_filename);
+		Conv.SetInAndOutFormats (pOutFormat, pOutFormat);
+		Conv.Write (&Mol, &ofs);
+#else
 		OBFileFormat fileFormat;
 		fileFormat.WriteMolecule(ofs, Mol, "2D");
+#endif
 		setlocale(LC_NUMERIC, old_num_locale);
 		g_free(old_num_locale);
-		ofs.close();
+		if ((res = gnome_vfs_write (handle, ofs.str ().c_str (), (GnomeVFSFileSize) ofs.str ().size (), &n)) != GNOME_VFS_OK)
+			throw (int) res;
+		gnome_vfs_close (handle);
+		SetReadOnly (false);
 	}
 	catch (int n)
 	{
+		printf("error #%d\n",n);
 	}
 }
 
@@ -322,7 +361,6 @@ void gcpDocument::AddAtom(gcpAtom* pAtom)
 	char id[8];
 	const gchar *Id;
 	Id = pAtom->GetId();
-	Object* pObject = GetDescendant(Id);
 	if (Id == NULL) 
 	{
 		id[0] = 'a';
@@ -332,13 +370,10 @@ void gcpDocument::AddAtom(gcpAtom* pAtom)
 		pAtom->SetId(id);
 		Id = id;
 	}
+	m_pView->AddObject(pAtom);
 	if (m_bIsLoading)
-	{
-		m_pView->AddObject(pAtom);
 		return;
-	}
-	AddObject(pAtom);
-	int i0 = 1;
+//	AddObject(pAtom);
 	gcpMolecule* mol = new gcpMolecule();
 	i = 1;
 	id[0] = 'm';
@@ -357,7 +392,6 @@ void gcpDocument::AddFragment(gcpFragment* pFragment)
 	char id[8];
 	const gchar *Id;
 	Id = pFragment->GetId();
-	Object* pObject = GetDescendant(Id);
 	if (Id == NULL) 
 	{
 		id[0] = 'f';
@@ -367,16 +401,12 @@ void gcpDocument::AddFragment(gcpFragment* pFragment)
 		pFragment->SetId(id);
 		Id = id;
 	}
+	m_pView->AddObject(pFragment);
 	if (m_bIsLoading)
-	{
-		m_pView->AddObject(pFragment);
 		return;
-	}
-	AddObject(pFragment);
-	pFragment->AnalContent();
+//	AddObject(pFragment);
 	if (!pFragment->GetMolecule())
 	{
-		int i0 = 1;
 		gcpMolecule* mol = new gcpMolecule();
 		i = 1;
 		id[0] = 'm';
@@ -387,6 +417,7 @@ void gcpDocument::AddFragment(gcpFragment* pFragment)
 		AddChild(mol);
 		mol->AddFragment(pFragment);
 	}
+	pFragment->AnalContent();
 	m_bDirty = true;
 }
 
@@ -396,7 +427,6 @@ void gcpDocument::AddBond(gcpBond* pBond)
 	char id[8];
 	const gchar *Id;
 	Id = pBond->GetId();
-	Object* pObject = GetDescendant(Id);
 	if (Id == NULL)
 	{
 		id[0] = 'b';
@@ -408,12 +438,10 @@ void gcpDocument::AddBond(gcpBond* pBond)
 	gcpAtom *pAtom0 = (gcpAtom*)pBond->GetAtom(0), *pAtom1 = (gcpAtom*)pBond->GetAtom(1);
 	m_pView->Update(pAtom0);
 	m_pView->Update(pAtom1);
+	m_pView->AddObject(pBond);
 	if (m_bIsLoading)
-	{
-		m_pView->AddObject(pBond);
 		return;
-	}
-	AddObject(pBond);
+//	AddObject(pBond);
 	//search molecules
 	gcpMolecule * pMol0 = (gcpMolecule*)pAtom0->GetMolecule(),  *pMol1 = (gcpMolecule*)pAtom1->GetMolecule();
 	if (pMol0 && pMol1)
@@ -451,8 +479,12 @@ void gcpDocument::AddBond(gcpBond* pBond)
 
 void gcpDocument::Save()
 {
-	if (!m_filename || !m_bWriteable) return;
+	if (!m_filename || !m_bWriteable || m_bReadOnly) return;
+#ifdef HAS_OPENBABEL_2
+	if (m_FileType != gcp_ft_Normal)
+#else
 	if (m_FileType != UNDEFINED)
+#endif
 	{
 		ExportOB();
 	}
@@ -461,52 +493,83 @@ void gcpDocument::Save()
 	m_bDirty = false;
 }
 
-bool gcpDocument::Load(xmlNodePtr root)
+bool gcpDocument::Load (xmlNodePtr root)
 {
-	if (m_title) {g_free(m_title) ; m_title = NULL;}
-	if (m_author) {g_free(m_author) ; m_author = NULL;}
-	if (m_mail) {g_free(m_mail) ; m_mail = NULL;}
-	if (m_comment) {g_free(m_comment) ; m_comment = NULL;}
-	g_date_clear(&CreationDate, 1);
-	g_date_clear(&RevisionDate, 1);
+	if (m_title) {
+		g_free(m_title);
+		m_title = NULL;
+	}
+	if (m_author) {
+		g_free(m_author);
+		m_author = NULL;
+	}
+	if (m_mail) {
+		g_free(m_mail);
+		m_mail = NULL;
+	}
+	if (m_comment) {
+		g_free(m_comment);
+		m_comment = NULL;
+	}
+	g_date_clear (&CreationDate, 1);
+	g_date_clear (&RevisionDate, 1);
+#ifdef HAS_OPENBABEL_2
+	m_FileType = gcp_ft_Normal;
+#else
 	m_FileType = UNDEFINED;
+#endif
 	xmlNodePtr node, child;
 	char* tmp;
 	Object* pObject;
-	tmp = (char*)xmlGetProp(root, (xmlChar*)"id");
-	if (tmp) SetId(tmp);
-	tmp = (char*)xmlGetProp(root, (xmlChar*)"creation");
-	if (tmp)
-	{
+	tmp = (char*) xmlGetProp (root, (xmlChar*) "id");
+	if (tmp) {
+		SetId (tmp);
+		xmlFree (tmp);
+	}
+	tmp = (char*) xmlGetProp (root, (xmlChar*) "creation");
+	if (tmp) {
 		g_date_set_parse(&CreationDate, tmp);
 		if (!g_date_valid(&CreationDate)) g_date_clear(&CreationDate, 1);
+		xmlFree (tmp);
 	}
-	tmp = (char*)xmlGetProp(root, (xmlChar*)"revision");
-	if (tmp)
-	{
+	tmp = (char*) xmlGetProp (root, (xmlChar*) "revision");
+	if (tmp) {
 		g_date_set_parse(&RevisionDate, tmp);
 		if (!g_date_valid(&RevisionDate)) g_date_clear(&RevisionDate, 1);
+		xmlFree (tmp);
 	}
 
-	node = GetNodeByName(root, "title");
+	node = GetNodeByName (root, "title");
 	if (node)
 	{
-		tmp = (char*)xmlNodeGetContent(node);
-		if (tmp) m_title = g_strdup(tmp);
+		tmp = (char*) xmlNodeGetContent (node);
+		if (tmp) {
+			m_title = g_strdup(tmp);
+			xmlFree (tmp);
+		}
 	}
-	node = GetNodeByName(root, "author");
+	node = GetNodeByName (root, "author");
 	if (node)
 	{
-		tmp = (char*)xmlGetProp(node, (xmlChar*)"name");
-		if (tmp) m_author = g_strdup(tmp);
-		tmp = (char*)xmlGetProp(node, (xmlChar*)"e-mail");
-		if (tmp) m_mail = g_strdup(tmp);
+		tmp = (char*) xmlGetProp (node, (xmlChar*) "name");
+		if (tmp) {
+			m_author = g_strdup(tmp);
+			xmlFree (tmp);
+		}
+		tmp = (char*) xmlGetProp (node, (xmlChar*) "e-mail");
+		if (tmp) {
+			m_mail = g_strdup(tmp);
+			xmlFree (tmp);
+		}
 	}
-	node = GetNodeByName(root, "comment");
+	node = GetNodeByName (root, "comment");
 	if (node)
 	{
-		tmp = (char*)xmlNodeGetContent(node);
-		if (tmp) m_comment = g_strdup(tmp);
+		tmp = (char*) xmlNodeGetContent (node);
+		if (tmp) {
+			m_comment = g_strdup(tmp);
+			xmlFree (tmp);
+		}
 	}
 
 	m_bIsLoading = true;
@@ -523,13 +586,22 @@ bool gcpDocument::Load(xmlNodePtr root)
 		node = node->next;
 	}
 	m_pView->Update (this);
+	Update ();
 	m_bIsLoading = false;
+	m_pApp->ActivateMenu ("Image", HasChildren ());
+	m_pView->EnsureSize ();
 	return true;
 }
 	
-void gcpDocument::ParseXMLTree(xmlDocPtr xml)
+void gcpDocument::ParseXMLTree (xmlDocPtr xml)
 {
-	Load(xml->children);
+	Load (xml->children);
+}
+
+static int cb_xml_to_vfs (GnomeVFSHandle *handle, const char* buf, int nb)
+{
+	GnomeVFSFileSize ndone;
+	return (int) gnome_vfs_write (handle, buf, nb, &ndone);
 }
 
 xmlDocPtr gcpDocument::BuildXMLTree()
@@ -551,7 +623,6 @@ xmlDocPtr gcpDocument::BuildXMLTree()
 	xml->children =  xmlNewDocNode(xml, NULL, (xmlChar*)"chemistry", NULL);
 	ns = xmlNewNs (xml->children, (xmlChar*) "http://www.nongnu.org/gchempaint", (xmlChar*) "gcp");
 	xmlSetNs (xml->children, ns);
-	SaveId(node);
 	if (!g_date_valid(&CreationDate))
 		g_date_set_time(&CreationDate, time(NULL));
 	g_date_set_time(&RevisionDate, time(NULL));
@@ -591,7 +662,30 @@ xmlDocPtr gcpDocument::BuildXMLTree()
 
 		xmlIndentTreeOutput = true;
 		xmlKeepBlanksDefault(0);
-		if (xmlSaveFormatFile(m_filename, xml, true) < 0) /*Error(SAVE)*/;
+
+		GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
+		gnome_vfs_get_file_info (m_filename, info, GNOME_VFS_FILE_INFO_DEFAULT);
+
+		if (GNOME_VFS_FILE_INFO_LOCAL (info)) {
+			gnome_vfs_file_info_unref (info);
+			if (xmlSaveFormatFile (m_filename, xml, true) < 0) /*Error(SAVE)*/;
+		} else {
+			gnome_vfs_file_info_unref (info);
+			xmlOutputBufferPtr buf = xmlAllocOutputBuffer (NULL);
+			GnomeVFSHandle *handle;
+			GnomeVFSResult result = gnome_vfs_open (&handle, m_filename, GNOME_VFS_OPEN_WRITE);
+			if (result == GNOME_VFS_ERROR_NOT_FOUND)
+				result = gnome_vfs_create (&handle, m_filename, GNOME_VFS_OPEN_WRITE, true, 0666);
+			if (result != GNOME_VFS_OK)
+				throw 1;
+			buf->context = handle;
+			buf->closecallback = (xmlOutputCloseCallback) gnome_vfs_close;
+			buf->writecallback = (xmlOutputWriteCallback) cb_xml_to_vfs;
+			int n = xmlSaveFormatFileTo (buf, xml, NULL, true);
+			if (n < 0)
+				throw 1;
+			SetReadOnly (false);
+		}
 //FIXME: the xml is freed and then we return it!!!
 		xmlFreeDoc(xml);
 		m_bDirty = false;
@@ -616,26 +710,24 @@ xmlDocPtr gcpDocument::BuildXMLTree()
 
 void gcpDocument::Update()
 {
-	std::list<Object*>::iterator i;
+	std::set<Object*>::iterator i,  end = m_DirtyObjects.end ();
 	TypeId Id;
-	for (i = m_DirtyObjects.begin(); i != m_DirtyObjects.end(); i++)
-	{
-		Id = (*i)->GetType();
-		switch (Id)
-		{
-			case BondType:
-				m_pView->Update((gcpBond*)(*i));
-				break;
+	for (i = m_DirtyObjects.begin(); i != end; i++) {
+		Id = (*i)->GetType ();
+		switch (Id) {
+		case BondType:
+			m_pView->Update ((gcpBond*)(*i));
+			break;
 		}
 	}
-	m_DirtyObjects.clear();
+	m_DirtyObjects.clear ();
 }
 	
 void gcpDocument::RemoveAtom(gcpAtom* pAtom)
 {
 	std::map<Atom*, Bond*>::iterator i;
 	gcpBond* pBond;
-	while (pBond = (gcpBond*)pAtom->GetFirstBond(i))
+	while ((pBond = (gcpBond*)pAtom->GetFirstBond(i)))
 	{
 		if (!m_bUndoRedo) m_pCurOp->AddObject(pBond);
 		RemoveBond(pBond);
@@ -651,7 +743,7 @@ void gcpDocument::RemoveFragment(gcpFragment* pFragment)
 	std::map<Atom*, Bond*>::iterator i;
 	gcpAtom* pAtom = pFragment->GetAtom();
 	gcpBond* pBond;
-	while (pBond = (gcpBond*)pAtom->GetFirstBond(i))
+	while ((pBond = (gcpBond*)pAtom->GetFirstBond(i)))
 	{
 		if (!m_bUndoRedo) m_pCurOp->AddObject(pBond);
 		RemoveBond(pBond);
@@ -662,59 +754,81 @@ void gcpDocument::RemoveFragment(gcpFragment* pFragment)
 	delete pFragment;
 }
 
-void gcpDocument::RemoveBond(gcpBond* pBond)
+void gcpDocument::RemoveBond (gcpBond* pBond)
 {
-	m_pView->Remove(pBond);
+	m_pView->Remove (pBond);
 	gcpAtom *pAtom0, *pAtom1;
-	pAtom0 = (gcpAtom*)pBond->GetAtom(0);
-	pAtom1 = (gcpAtom*)pBond->GetAtom(1);
-	gcpMolecule *pMol = (gcpMolecule*)pBond->GetMolecule();
+	pAtom0 = (gcpAtom*) pBond->GetAtom (0);
+	pAtom1 = (gcpAtom*) pBond->GetAtom (1);
+	gcpMolecule *pMol = (gcpMolecule*) pBond->GetMolecule ();
 	char id[16];
-	pAtom0->RemoveBond(pBond);
-	m_pView->Update(pAtom0);
-	pAtom1->RemoveBond(pBond);
-	m_pView->Update(pAtom1);
-	if (pBond->IsCyclic())
-	{
-		pMol->Remove(pBond);
-		pMol->UpdateCycles();
-		Update();
-	}
-	else
-	{
+	pMol->Lock (true);
+	pAtom0->RemoveBond (pBond);
+	m_pView->Update (pAtom0);
+	pAtom1->RemoveBond (pBond);
+	m_pView->Update (pAtom1);
+	pMol->Lock (false);
+	if (pBond->IsCyclic ()) {
+		pMol->Remove (pBond);
+		pMol->UpdateCycles ();
+		Update ();
+	} else {
+		Object *Parent = pMol->GetParent ();
+		Parent->Lock ();
 		int i0 = 1;
+		string align_id = pMol->GetAlignmentId ();
 		delete pMol;
-		gcpMolecule * pMol = new gcpMolecule();
-		AddChild(pMol);
-		do snprintf(id, sizeof(id), "m%d", i0++);
-		while (GetDescendant(id) != NULL);
-		pMol->SetId(id);
-		Object* pObject = pAtom0->GetParent();
-		if (pObject->GetType() == FragmentType) pMol->AddFragment((gcpFragment*)pObject);
-		else pMol->AddAtom(pAtom0);
-		pMol->UpdateCycles();
-		do snprintf(id, sizeof(id), "m%d", i0++);
-		while (GetDescendant(id) != NULL);
-		pMol = new gcpMolecule();
-		pMol->SetId(id);
-		AddChild(pMol);
-		pObject = pAtom1->GetParent();
-		if (pObject->GetType() == FragmentType) pMol->AddFragment((gcpFragment*)pObject);
-		else pMol->AddAtom(pAtom1);
-		pMol->UpdateCycles();
-		if ((pAtom0->GetZ() == 6) && (pAtom0->GetBondsNumber() == 0)) m_pView->Update(pAtom0);
-		if ((pAtom1->GetZ() == 6) && (pAtom1->GetBondsNumber() == 0)) m_pView->Update(pAtom1);
+		gcpMolecule * pMol = new gcpMolecule ();
+		pMol->Lock (true);
+		do
+			snprintf (id, sizeof (id), "m%d", i0++);
+		while (GetDescendant (id) != NULL);
+		pMol->SetId (id);
+		Parent->AddChild (pMol); /* do not update at this point it's the caller responsibility */
+		Object* pObject = pAtom0->GetParent ();
+		if (pObject->GetType () == FragmentType)
+			pMol->AddFragment ((gcpFragment*) pObject);
+		else
+			pMol->AddAtom (pAtom0);
+		pMol->UpdateCycles ();
+		if (align_id.size ()) {
+			Object *obj = pMol->GetDescendant (align_id.c_str ());
+			if (obj)
+				pMol->SelectAlignmentItem (obj);
+			align_id = "";
+		}
+		pMol->Lock (false);
+		do
+			snprintf (id, sizeof (id), "m%d", i0++);
+		while (GetDescendant (id) != NULL);
+		pMol = new gcpMolecule ();
+		pMol->Lock (true);
+		pMol->SetId (id);
+		Parent->AddChild (pMol); /* do not update at this point it's the caller responsibility */
+		pObject = pAtom1->GetParent ();
+		if (pObject->GetType () == FragmentType)
+			pMol->AddFragment ((gcpFragment*) pObject);
+		else
+			pMol->AddAtom (pAtom1);
+		pMol->UpdateCycles ();
+		if (align_id.size ()) {
+			Object *obj = pMol->GetDescendant (align_id.c_str ());
+			if (obj)
+				pMol->SelectAlignmentItem (obj);
+		}
+		pMol->Lock (false);
+		if ((pAtom0->GetZ () == 6) && (pAtom0->GetBondsNumber () == 0))
+			m_pView->Update (pAtom0);
+		if ((pAtom1->GetZ () == 6) && (pAtom1->GetBondsNumber () == 0))
+			m_pView->Update (pAtom1);
+		Parent->Lock (false);
 	}
+	m_DirtyObjects.erase (pBond);
 	delete pBond;
 }
 	
 void gcpDocument::Remove(Object* pObject)
 {
-	if (!m_bUndoRedo)
-	{
-		if (!m_pCurOp) m_pCurOp = new gcpDeleteOperation(this);
-		m_pCurOp->AddObject(pObject);
-	}
 	switch(pObject->GetType())
 	{
 		case AtomType:
@@ -746,6 +860,8 @@ void gcpDocument::Remove(Object* pObject)
 				Object* object = pObject->GetFirstChild(i);
 				while (object)
 				{
+					if (pObject->IsLocked ())
+						object->Lock ();
 					Remove(object);
 					object = pObject->GetFirstChild(i);
 				}
@@ -758,7 +874,10 @@ void gcpDocument::Remove(Object* pObject)
 void gcpDocument::Remove(const char* Id)
 {
 	Object* pObj = GetDescendant(Id);
-	if (pObj) Remove(pObj);
+	if (pObj) {
+		pObj->Lock ();
+		Remove(pObj);
+	}
 }
 
 void gcpDocument::OnProperties()
@@ -799,8 +918,10 @@ void gcpDocument::AddObject(Object* pObject)
 	if (!pObject->GetParent()) AddChild(pObject);
 	m_pView->AddObject(pObject);
 	if (m_bIsLoading || m_bUndoRedo) return;
-	if (!m_pCurOp) m_pCurOp = new gcpAddOperation(this);
-	m_pCurOp->AddObject(pObject);
+	if (!m_pCurOp) {
+		m_pCurOp = new gcpAddOperation(this);
+		m_pCurOp->AddObject(pObject);
+	}
 }
 
 void gcpDocument::OnUndo()
@@ -817,7 +938,10 @@ void gcpDocument::OnUndo()
 	}
 	if (m_UndoList.empty())
 		m_pApp->ActivateMenu("Undo", false);
+	m_pApp->ActivateMenu ("Image", HasChildren ());
 	m_bUndoRedo = false;
+	Update ();
+	EmptyTranslationTable ();
 }
 
 void gcpDocument::OnRedo()
@@ -834,7 +958,9 @@ void gcpDocument::OnRedo()
 	}
 	if (m_RedoList.empty())
 		m_pApp->ActivateMenu("Redo", false);
+	m_pApp->ActivateMenu ("Image", HasChildren ());
 	m_bUndoRedo = false;
+	EmptyTranslationTable ();
 }
 
 void gcpDocument::FinishOperation()
@@ -850,6 +976,8 @@ void gcpDocument::FinishOperation()
 	m_pApp->ActivateMenu("Redo", false);
 	m_pCurOp = NULL;
 	m_bDirty = true;
+	m_pApp->ActivateMenu ("Image", HasChildren ());
+	Update ();
 }
 
 void gcpDocument::AbortOperation()
@@ -889,10 +1017,13 @@ void gcpDocument::PushOperation(gcpOperation* operation, bool undo)
 	m_pCurOp = NULL;
 }
 
-void gcpDocument::SetActive()
+void gcpDocument::SetActive ()
 {
-	m_pApp->ActivateMenu("Undo", !m_UndoList.empty());
-	m_pApp->ActivateMenu("Redo", !m_RedoList.empty());
+	m_pApp->ActivateMenu("Undo", !m_UndoList.empty ());
+	m_pApp->ActivateMenu("Redo", !m_RedoList.empty ());
+	m_pApp->ActivateMenu("Image", HasChildren ());
+	m_pApp->ActivateMenu ("Save", !m_bReadOnly);
+	m_pApp->ActivateToolItem ("Save", !m_bReadOnly);
 }
 
 extern xmlDocPtr pXmlDoc;
@@ -906,6 +1037,7 @@ void gcpDocument::LoadObjects(xmlNodePtr node)
 		if (!strcmp((const char*)child->name, "atom"))
 		{
 			gcpAtom* pAtom = new gcpAtom();
+			AddChild (pAtom);
 			pAtom->Load(child);
 			AddAtom(pAtom);
 		}
@@ -923,7 +1055,7 @@ void gcpDocument::LoadObjects(xmlNodePtr node)
 			child1 = (strcmp((const char*)child->name, "object"))? child: child->children;
 			str = (const char*)child1->name;
 			Object* pObject = Object::CreateObject(str, this);
-			pObject->Load(child1); 
+			pObject->Load(child1);
 			AddObject(pObject);
 			m_pView->Update(pObject);//FIXME: should not be necessary, but solve problem with cyclic double bonds
 			m_bIsLoading = false;
@@ -953,6 +1085,8 @@ gcpOperation* gcpDocument::GetNewOperation(gcpOperationType type)
 			return m_pCurOp = new gcpDeleteOperation(this);
 		case GCP_MODIFY_OPERATION:
 			return m_pCurOp = new gcpModifyOperation(this);
+		default:
+			return NULL;
 	}
 }
 
@@ -983,3 +1117,31 @@ void gcpDocument::AddData(xmlNodePtr node)
 	EmptyTranslationTable();
 	FinishOperation();
 }
+
+void gcpDocument::ExportImage (const char* filename, const char* type)
+{
+	m_pView->ExportImage (filename, type);
+}
+
+void gcpDocument::SetReadOnly (bool ro)
+{
+	m_bReadOnly = ro;
+	if (m_pApp->GetActiveDocument () == this) {
+		m_pApp->ActivateMenu ("Save", !ro);
+		m_pApp->ActivateToolItem ("Save", !ro);
+	}
+}
+
+double gcpDocument::GetYAlign ()
+{
+	if (GetChildrenNumber () == 1) {
+		map<string, Object*>::iterator i;
+		Object *Child = GetFirstChild (i);
+		return Child->GetYAlign ();
+	} else {
+		gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (GetWidget ()), "data");
+		ArtDRect rect;
+		pData->GetObjectBounds (this, &rect);
+		return (rect.y1 - rect.y0) / 2.;
+	}
+}
