@@ -32,6 +32,7 @@
 #include <sys/stat.h>
 
 gcpTemplatesPlugin plugin;
+extern set<xmlDocPtr> docs;
 
 gcpTemplatesPlugin::gcpTemplatesPlugin(): gcpPlugin()
 {
@@ -44,10 +45,11 @@ gcpTemplatesPlugin::gcpTemplatesPlugin(): gcpPlugin()
 	xmlIndentTreeOutput = true;
 	xmlKeepBlanksDefault (0);
 	if (dir) {
-		while (name = g_dir_read_name (dir)) {
+		while ((name = g_dir_read_name (dir))) {
 			if (strcmp (name + strlen (name) - 4, ".xml")) continue;
 			name = g_strconcat (DATADIR"/gchempaint/templates/", name, NULL);
 			doc = xmlParseFile (name);
+			docs.insert (doc);
 			node = doc->children;
 			if (!strcmp ((char*)node->name, "templates")) ParseNodes (node->children, false);
 			g_free ((void*) name);
@@ -58,11 +60,12 @@ gcpTemplatesPlugin::gcpTemplatesPlugin(): gcpPlugin()
 	dir = g_dir_open (path, 0, NULL);
 	if (dir) {
 		bool is_user_templates;
-		while (name = g_dir_read_name (dir)) {
+		while ((name = g_dir_read_name (dir))) {
 			if (strcmp (name + strlen (name) - 4, ".xml")) continue;
 			is_user_templates = !strcmp (name, "templates.xml");
 			name = g_strconcat (path, "/", name, NULL);
 			doc = xmlParseFile (name);
+			docs.insert (doc);
 			if (is_user_templates)
 				user_templates = doc;
 			node = doc->children;
@@ -85,6 +88,15 @@ gcpTemplatesPlugin::gcpTemplatesPlugin(): gcpPlugin()
 
 gcpTemplatesPlugin::~gcpTemplatesPlugin()
 {
+	set<xmlDocPtr>::iterator i, iend = docs.end ();
+	for (i = docs.begin (); i != iend; i++)
+		xmlFreeDoc (*i);
+	docs.clear ();
+	map<string, gcpTemplate*>::iterator j, jend = Templates.end ();
+	for (j = Templates.begin (); j != jend; j++)
+		delete (*j).second;
+	Templates.clear ();
+	TempbyName.clear ();
 }
 
 void gcpTemplatesPlugin::Populate (gcpApplication* App)
